@@ -15,6 +15,7 @@
 from functools import partial
 import json
 import os
+import signal
 
 from storlets.sbus import SBus
 import storlets.sbus.command as sbus_cmd
@@ -22,6 +23,9 @@ import storlets.sbus.command as sbus_cmd
 EXIT_SUCCESS = 0
 EXIT_FAILURE = 1
 
+LOOP_TIMEOUT = 0.5
+LISTEN_TIMEOUT = 300
+
 
 class CommandResponse(Exception):
     """
@@ -30,11 +34,11 @@ class CommandResponse(Exception):
 
     def __init__(self, status, message, iterable=True, task_id=None):
         """
-        Constract CommandResponse instance
+        Construct CommandResponse instance
 
         :param status: task status
         :param message: message to be returned and logged
-        :param iterable: wheter we can keep SDaemon process running
+        :param iterable: whether we can keep SDaemon process running
         :param task_id: ID assigned to the requested task
         """
         self.status = status
@@ -77,6 +81,9 @@ class SBusServer(object):
     def __init__(self, sbus_path, logger):
         self.sbus_path = sbus_path
         self.logger = logger
+        self.running = True
+        self.listen_timeout = LISTEN_TIMEOUT
+        self.loop_timeout = LOOP_TIMEOUT
 
     def get_handler(self, command):
         """
@@ -155,12 +162,29 @@ class SBusServer(object):
     def _terminate(self):
         raise NotImplementedError()
 
+    def _force_terminate(self):
+        raise NotImplementedError()
+
+    def _graceful_shutdown(self, *args):
+        self.logger.info("received SIGHUP. Shutting down gracefully")
+        self.running = False
+
+    def _force_shutdown(self, *args):
+        self.logger.info("received SIGTERM. Shutting down now")
+        try:
+            self._force_terminate()
+        except Exception:
+            self.logger.exception(
+                "Failed to force_terminate. The process stops anyway")
+        finally:
+            os._exit(1)
+
     def main_loop(self):
         """
         Main loop to run storlet application
 
         :returns: EXIT_SUCCESS when the loop exists normally
-                  EXIT_FAILURE when some error occurd in main loop
+                  EXIT_FAILURE when an error occurs in main loop
         """
         sbus = SBus()
         fd = sbus.create(self.sbus_path)
@@ -168,20 +192,39 @@ class SBusServer(object):
             self.logger.error("Failed to create SBus. exiting.")
             return EXIT_FAILURE
 
-        while True:
-            rc = sbus.listen(fd)
+        signal.signal(signal.SIGHUP, self._graceful_shutdown)
+        signal.signal(signal.SIGTERM, self._force_shutdown)
+
+        loop_cnt = 0
+        status = EXIT_SUCCESS
+
+        while self.running:
+            rc = sbus.listen(fd, self.loop_timeout)
+
             if rc < 0:
                 self.logger.error("Failed to wait on SBus. exiting.")
-                return EXIT_FAILURE
+                status = EXIT_FAILURE
+                break
+            elif rc == 0:
+                loop_cnt += 1
+                if loop_cnt * self.loop_timeout >= self.listen_timeout:
+                    self.logger.debug("Timed out while listening. exiting.")
+                    break
+                continue
+
+            # Reset loop_cnt here as a request comes via sbus
+            loop_cnt = 0
 
             dtg = sbus.receive(fd)
             if dtg is None:
                 self.logger.error("Failed to receive message. exiting")
-                return EXIT_FAILURE
+                status = EXIT_FAILURE
+                break
 
             if not self.dispatch_command(dtg):
                 break
 
         self.logger.debug('Leaving main loop')
         self._terminate()
-        return EXIT_SUCCESS
+
+        return status
