@@ -81,12 +81,12 @@ class StorletFile(object):
 
 
 class StorletOutputFile(StorletFile):
-    mode = 'w'
+    mode = 'wb'
 
     def __init__(self, md_fd, obj_fd):
         super(StorletOutputFile, self).__init__(obj_fd)
         self._metadata = None
-        self.md_file = os.fdopen(md_fd, 'w')
+        self.md_file = os.fdopen(md_fd, 'wb')
 
     def get_metadata(self):
         if self._metadata is None:
@@ -96,7 +96,7 @@ class StorletOutputFile(StorletFile):
     def set_metadata(self, md):
         if self.md_file.closed:
             raise IOError('Sending metadata twice is not allowed')
-        self.md_file.write(json.dumps(md))
+        self.md_file.write(json.dumps(md).encode('utf-8'))
         self._metadata = md
         self.md_file.close()
 
@@ -180,7 +180,7 @@ class StorletRangeInputFile(StorletInputFile):
         self.start = start
         self.end = end
         # TODO(takashi): Currently we use range input file only for zero copy
-        #                case, so can execute seek on fd. Myabe we need some
+        #                case, so can execute seek on fd. Maybe we need some
         #                mechanism to confirm the fd is seekable.
         self.obj_file.seek(self.start, 0)
         self.point = self.start
@@ -201,7 +201,7 @@ class StorletLogger(object):
     def __init__(self, storlet_name, fd):
         self.fd = fd
         self.storlet_name = storlet_name
-        self.log_file = os.fdopen(self.fd, 'w')
+        self.log_file = os.fdopen(self.fd, 'wb')
 
     @property
     def closed(self):
@@ -218,7 +218,7 @@ class StorletLogger(object):
 
     def _emit_log(self, level, msg):
         msg = '%s %s: %s' % (self.storlet_name, level, msg)
-        self.log_file.write(msg)
+        self.log_file.write(msg.encode('utf-8'))
 
     def debug(self, msg):
         self._emit_log('DEBUG', msg)
