@@ -26,7 +26,7 @@ from storlets.sbus.client.exceptions import SBusClientException, \
     SBusClientSendError
 from storlets.agent.common.server import command_handler, EXIT_FAILURE, \
     CommandSuccess, CommandFailure, SBusServer
-from storlets.agent.common.utils import get_logger, DEFAULT_PY3
+from storlets.agent.common.utils import get_logger
 
 
 class SDaemonError(Exception):
@@ -96,16 +96,13 @@ class StorletDaemonFactory(SBusServer):
     def get_python_args(self, daemon_language, storlet_path, storlet_name,
                         pool_size, uds_path, log_level,
                         daemon_language_version):
-        # TODO(takashi): We currently override all 3.x by DEFAULT_PY3, but
-        #                we might need to support multiple minor versions.
-        daemon_language_version = DEFAULT_PY3
-
+        daemon_language_version = daemon_language_version or '3'
         python_interpreter = '/usr/bin/python%s' % daemon_language_version
         str_daemon_main_file = '/usr/local/libexec/storlets/storlets-daemon'
         pargs = [python_interpreter, str_daemon_main_file, storlet_name,
                  uds_path, log_level, str(pool_size), self.container_id]
 
-        python_path = os.path.join('/home/swift/', storlet_name)
+        python_path = os.path.join('/var/lib/storlets', storlet_name)
         if os.environ.get('PYTHONPATH'):
             python_path = os.environ['PYTHONPATH'] + ':' + python_path
         env = {'PYTHONPATH': python_path}
@@ -126,22 +123,20 @@ class StorletDaemonFactory(SBusServer):
         if not pargs:
             raise SDaemonError('Invalid process arguments')
 
-        if not (os.path.exists(pargs[0]) and os.access(pargs[0], os.X_OK)):
+        if not (os.path.isfile(pargs[0]) and os.access(pargs[0], os.X_OK)):
             raise SDaemonError('Requested runtime is unavailable')
 
         str_pargs = ' '.join(pargs)
         self.logger.debug('Starting subprocess: pargs:{0} env:{1}'
                           .format(str_pargs, env))
 
-        # TODO(takashi): We had better use contextmanager
-        # TODO(takashi): Where is this closed?
         try:
-            dn = open(os.devnull, 'wb')
             daemon_p = subprocess.Popen(
-                pargs, stdout=dn, stderr=subprocess.PIPE,
+                pargs, stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
                 close_fds=True, shell=False, env=env)
             logger_p = subprocess.Popen(
-                'logger', stdin=daemon_p.stderr, stdout=dn, stderr=dn,
+                'logger', stdin=daemon_p.stdout,
+                stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL,
                 close_fds=True, shell=False)
         except OSError:
             self.logger.exception('Unable to start subprocess')
@@ -339,7 +334,7 @@ class StorletDaemonFactory(SBusServer):
         """
         Kill every one.
 
-        :param try_all: wheather we try to kill all process if we fail to
+        :param try_all: whether we try to kill all process if we fail to
                         stop some of the storlet daemons
         :raises SDaemonError: when failed to kill one of the storlet daemons
         """
@@ -364,7 +359,7 @@ class StorletDaemonFactory(SBusServer):
         """
         send HALT command to every spawned process
 
-        :param try_all: wheather we try to kill all process if we fail to
+        :param try_all: whether we try to kill all process if we fail to
                         stop some of the storlet daemons
         :returns: a list of the terminated storlet daemons
         :raises SDaemonError: when failed to kill one of the storlet daemons
@@ -534,7 +529,8 @@ def main():
     logger.debug("Daemon factory started")
 
     try:
-        SBus.start_logger("DEBUG", container_id=opts.container_id)
+        SBus.start_logger(opts.log_level.upper(),
+                          container_id=opts.container_id)
         # create an instance of daemon_factory
         factory = StorletDaemonFactory(opts.sbus_path, logger,
                                        opts.container_id)
@@ -543,5 +539,5 @@ def main():
         sys.exit(factory.main_loop())
 
     except Exception:
-        logger.eception('Unhandled exception')
+        logger.exception('Unhandled exception')
         sys.exit(EXIT_FAILURE)
