@@ -12,72 +12,73 @@
 # implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+import abc
 import copy
 import json
 
+from storlets.sbus import command as sbus_cmd
 from storlets.sbus import file_description as sbus_fd
-from storlets.sbus.command import SBUS_CMD_EXECUTE
 
 
-class FDMetadata(object):
-    def __init__(self, fdtype, storlets_metadata=None, storage_metadata=None):
+class SBusFileDescriptor(object):
+    """
+    The management class for the file descriptor information
+    """
+
+    def __init__(self, fdtype, fileno, storlets_metadata=None,
+                 storage_metadata=None):
         self.fdtype = fdtype
+        self.fileno = fileno
         self.storlets_metadata = storlets_metadata or {}
         self.storage_metadata = storage_metadata or {}
 
-    def to_dict(self):
+    @property
+    def metadata(self):
         storlets_metadata = copy.deepcopy(self.storlets_metadata)
         storlets_metadata['type'] = self.fdtype
         return {'storlets': storlets_metadata,
                 'storage': self.storage_metadata}
 
     @classmethod
-    def from_dict(cls, metadict):
+    def from_fileno_and_metadata_dict(cls, fileno, metadict):
         _metadict = copy.deepcopy(metadict)
         storlets_metadata = _metadict['storlets']
         storage_metadata = _metadict['storage']
         fdtype = storlets_metadata.pop('type')
-        return cls(fdtype, storlets_metadata, storage_metadata)
+        return cls(fdtype, fileno, storlets_metadata, storage_metadata)
 
 
-class SBusDatagram(object):
+class SBusDatagram(object, metaclass=abc.ABCMeta):
     """
     The manager class for the datagram passed over sbus protocol
     """
 
-    # Each child Datagram should define what fd types are expected with
-    # list format
-    _required_fd_types = None
+    @property
+    @abc.abstractmethod
+    def _required_fdtypes(self):
+        pass
 
-    def __init__(self, command, fds, metadata, params=None, task_id=None):
+    def __init__(self, command, sfds, params=None, task_id=None):
         """
         Create SBusDatagram instance
 
         :param command: A string encoding the command to send
-        :param fds: A list of file descriptors (integer) to pass with
-                    the command
-        :param metadata: A list of dictionaries, where the i'th dictionary
-                          is the metadata of the i'th fd.
+        :param sfds: A list of SBusFileDescriptor instances
         :param params: A optional dictionary with parameters for the command
                        execution
         :param task_id: An optional string task id. This is currently used for
                         cancel command
         """
-        if type(self) == SBusDatagram:
-            raise NotImplementedError(
-                'SBusDatagram class should not be initialized as bare')
         self.command = command
-        self._check_fd_nums(fds, metadata)
-        fd_types = [md['storlets']['type'] for md in metadata]
-        self._check_required_fd_types(fd_types)
-        self.fds = fds
-        self.metadata = metadata
+        fdtypes = [sfd.fdtype for sfd in sfds]
+        self._check_required_fdtypes(fdtypes)
+        self.sfds = sfds
         self.params = params
         self.task_id = task_id
 
     @property
     def num_fds(self):
-        return len(self.fds)
+        return len(self.sfds)
 
     @property
     def cmd_params(self):
@@ -92,19 +93,30 @@ class SBusDatagram(object):
     def serialized_cmd_params(self):
         return json.dumps(self.cmd_params)
 
+    @property
+    def fds(self):
+        """
+        A list of raw file descriptors
+        """
+        return [sfd.fileno for sfd in self.sfds]
+
+    @property
+    def metadata(self):
+        return [sfd.metadata for sfd in self.sfds]
+
     @property
     def serialized_metadata(self):
         return json.dumps(self.metadata)
 
     @property
     def object_in_metadata(self):
-        return [md['storage'] for md in self.metadata
-                if md['storlets']['type'] == sbus_fd.SBUS_FD_INPUT_OBJECT]
+        return [sfd.storage_metadata for sfd in self.sfds
+                if sfd.fdtype == sbus_fd.SBUS_FD_INPUT_OBJECT]
 
     @property
     def object_in_storlet_metadata(self):
-        return [md['storlets'] for md in self.metadata
-                if md['storlets']['type'] == sbus_fd.SBUS_FD_INPUT_OBJECT]
+        return [fd.storlets_metadata for fd in self.sfds
+                if fd.fdtype == sbus_fd.SBUS_FD_INPUT_OBJECT]
 
     def _find_fds(self, fdtype):
         """
@@ -113,11 +125,7 @@ class SBusDatagram(object):
         :param fdtype: file descriptor type
         :returns: a list of file descriptors
         """
-        ret = []
-        for i in range(len(self.metadata)):
-            if self.metadata[i]['storlets']['type'] == fdtype:
-                ret.append(self.fds[i])
-        return ret
+        return [sfd.fileno for sfd in self.sfds if sfd.fdtype == fdtype]
 
     def _find_fd(self, fdtype):
         """
@@ -135,22 +143,12 @@ class SBusDatagram(object):
             #                fd validation.
             return ret[0]
 
-    def _check_fd_nums(self, fds, metadata):
-        if len(fds) != len(metadata):
-            raise ValueError('Length mismatch fds:%s metadata:%s'
-                             % (len(fds), len(metadata)))
-
-    def _check_required_fd_types(self, given_fd_types):
-        if self._required_fd_types is None:
-            raise NotImplementedError(
-                'SBusDatagram class should define _required_fd_types')
-        # the first len(self._required_fd_types) types should be fit
-        # to the required list
-        if given_fd_types[:len(self._required_fd_types)] != \
-                self._required_fd_types:
-            raise ValueError('Fd type mismatch given_fd_types:%s \
-                             required_fd_types:%s' %
-                             (given_fd_types, self._required_fd_types))
+    def _check_required_fdtypes(self, given_fdtypes):
+        if given_fdtypes[:len(self._required_fdtypes)] != \
+                self._required_fdtypes:
+            raise ValueError('Fd type mismatch given_fdtypes:%s \
+                             required_fdtypes:%s' %
+                             (given_fdtypes, self._required_fdtypes))
 
     def __str__(self):
         return 'num_fds=%s, md=%s, cmd_params=%s' % (
@@ -174,11 +172,13 @@ class SBusServiceDatagram(SBusDatagram):
      - SBUS_CMD_PING
      - SBUS_CMD_CANCEL
     """
-    _required_fd_types = [sbus_fd.SBUS_FD_SERVICE_OUT]
+    @property
+    def _required_fdtypes(self):
+        return [sbus_fd.SBUS_FD_SERVICE_OUT]
 
-    def __init__(self, command, fds, metadata, params=None, task_id=None):
+    def __init__(self, command, sfds, params=None, task_id=None):
         super(SBusServiceDatagram, self).__init__(
-            command, fds, metadata, params, task_id)
+            command, sfds, params, task_id)
 
     @property
     def service_out_fd(self):
@@ -186,13 +186,18 @@ class SBusServiceDatagram(SBusDatagram):
 
 
 class SBusExecuteDatagram(SBusDatagram):
-    _required_fd_types = [sbus_fd.SBUS_FD_INPUT_OBJECT,
-                          sbus_fd.SBUS_FD_OUTPUT_TASK_ID,
-                          sbus_fd.SBUS_FD_OUTPUT_OBJECT,
-                          sbus_fd.SBUS_FD_OUTPUT_OBJECT_METADATA,
-                          sbus_fd.SBUS_FD_LOGGER]
 
-    def __init__(self, command, fds, metadata, params=None, task_id=None):
+    @property
+    def _required_fdtypes(self):
+        return [
+            sbus_fd.SBUS_FD_SERVICE_OUT,
+            sbus_fd.SBUS_FD_INPUT_OBJECT,
+            sbus_fd.SBUS_FD_OUTPUT_OBJECT,
+            sbus_fd.SBUS_FD_OUTPUT_OBJECT_METADATA,
+            sbus_fd.SBUS_FD_LOGGER
+        ]
+
+    def __init__(self, command, sfds, params=None, task_id=None):
         # TODO(kota_): the args command is not used in ExecuteDatagram
         #              but it could be worthful to taransparent init
         #              for other datagram classes.
@@ -202,16 +207,23 @@ class SBusExecuteDatagram(SBusDatagram):
         #                this implementation is based on the idea that
         #                we only have extra input sources, which is
         #                added at the end of fd list
-        extra_fd_types = [
-            md['storlets']['type'] == sbus_fd.SBUS_FD_INPUT_OBJECT
-            for md in metadata[len(self._required_fd_types):]]
+        extra_fdtypes = [sfd.fdtype for sfd in
+                         sfds[len(self._required_fdtypes):]]
 
-        if not all(extra_fd_types):
+        if [t for t in extra_fdtypes if t != sbus_fd.SBUS_FD_INPUT_OBJECT]:
             raise ValueError(
-                'Extra data should be SBUS_FD_INPUT_OBJECT: %' % metadata)
+                'Extra data should be SBUS_FD_INPUT_OBJECT')
 
         super(SBusExecuteDatagram, self).__init__(
-            SBUS_CMD_EXECUTE, fds, metadata, params, task_id)
+            sbus_cmd.SBUS_CMD_EXECUTE, sfds, params, task_id)
+
+    @property
+    def invocation_fds(self):
+        return [sfd.fileno for sfd in self.sfds[1:]]
+
+    @property
+    def service_out_fd(self):
+        return self._find_fd(sbus_fd.SBUS_FD_SERVICE_OUT)
 
     @property
     def object_out_fds(self):
@@ -221,10 +233,6 @@ class SBusExecuteDatagram(SBusDatagram):
     def object_metadata_out_fds(self):
         return self._find_fds(sbus_fd.SBUS_FD_OUTPUT_OBJECT_METADATA)
 
-    @property
-    def task_id_out_fd(self):
-        return self._find_fd(sbus_fd.SBUS_FD_OUTPUT_TASK_ID)
-
     @property
     def logger_out_fd(self):
         return self._find_fd(sbus_fd.SBUS_FD_LOGGER)
@@ -234,6 +242,15 @@ class SBusExecuteDatagram(SBusDatagram):
         return self._find_fds(sbus_fd.SBUS_FD_INPUT_OBJECT)
 
 
+def build_datagram(command, sfds, params, task_id):
+    if command == sbus_cmd.SBUS_CMD_EXECUTE:
+        dtg_cls = SBusExecuteDatagram
+    else:
+        dtg_cls = SBusServiceDatagram
+
+    return dtg_cls(command, sfds, params, task_id)
+
+
 def build_datagram_from_raw_message(fds, str_md, str_cmd_params):
     """
     Build SBusDatagram from raw message received over sbus
@@ -248,6 +265,10 @@ def build_datagram_from_raw_message(fds, str_md, str_cmd_params):
     command = cmd_params.get('command')
     params = cmd_params.get('params')
     task_id = cmd_params.get('task_id')
-    if command == SBUS_CMD_EXECUTE:
-        return SBusExecuteDatagram(command, fds, metadata, params, task_id)
-    return SBusServiceDatagram(command, fds, metadata, params, task_id)
+
+    if len(fds) != len(metadata):
+        raise ValueError('Length mismatch fds: %d != md %d' %
+                         (len(fds), len(metadata)))
+    sfds = [SBusFileDescriptor.from_fileno_and_metadata_dict(fileno, md)
+            for (fileno, md) in zip(fds, metadata)]
+    return build_datagram(command, sfds, params, task_id)
