@@ -16,22 +16,22 @@
 import os
 import tempfile
 from swiftclient import client
-from nose.plugins.attrib import attr
 from tests.functional.python import StorletPythonFunctionalTest
 import unittest
-from eventlet.green import urllib2
+from urllib.request import Request, urlopen
 
 
 class TestSimpleStorlet(StorletPythonFunctionalTest):
-    def setUp(self):
+    def setUp(self, version=None):
         self.storlet_log = 'simple.log'
-        self.content = 'abcdefghijklmonp'
+        self.content = b'abcdefghijklmonp'
         self.additional_headers = {}
         super(TestSimpleStorlet, self).setUp(
             storlet_dir='simple',
             storlet_name='simple.py',
             storlet_main='simple.SimpleStorlet',
-            storlet_file='source.txt')
+            storlet_file='source.txt',
+            version=version)
 
     def test_get(self):
         resp = dict()
@@ -78,26 +78,27 @@ class TestSimpleStorlet(StorletPythonFunctionalTest):
             response_dict=resp)
         self.assertEqual(204, resp['status'])
 
-    @attr('slow')
-    def test_put_512MB_file(self):
+    def test_put_10MB_file(self):
         with tempfile.NamedTemporaryFile() as f:
-            with open(f.name, 'w') as wf:
-                for _ in range(128):
-                    wf.write('1' * (4 * 1024 * 1024))
+            with open(f.name, 'wb') as wf:
+                for _ in range(10):
+                    wf.write(b'1' * (1 * 1024 * 1024))
 
             headers = {'X-Run-Storlet': self.storlet_name}
             headers.update(self.additional_headers)
-            with open(f.name, 'r') as rf:
+            with open(f.name, 'rb') as rf:
                 response = dict()
                 client.put_object(self.url, self.token,
                                   self.container, f.name, rf,
-                                  512 * 1024 * 1024, None, None,
+                                  10 * 1024 * 1024, None, None,
                                   "application/octet-stream",
                                   headers, None, None, None, response)
 
             status = response.get('status')
             self.assertEqual(201, status)
 
+    test_put_10MB_file.slow = 1
+
     def test_copy_from(self):
         resp = dict()
         objname = self.storlet_file + '-copy'
@@ -134,9 +135,9 @@ class TestSimpleStorlet(StorletPythonFunctionalTest):
                    'X-Run-Storlet': self.storlet_name,
                    'Destination': '%s/%s' % (self.container, objname)}
         headers.update(self.additional_headers)
-        req = urllib2.Request(url, headers=headers)
+        req = Request(url, headers=headers)
         req.get_method = lambda: 'COPY'
-        conn = urllib2.urlopen(req, timeout=10)
+        conn = urlopen(req, timeout=10)
 
         self.assertEqual(201, conn.getcode())
         self.assertEqual('%s/%s' % (self.container, self.storlet_file),
