@@ -13,9 +13,9 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import mock
-import unittest
 import itertools
+import unittest
+from unittest import mock
 
 from contextlib import contextmanager
 from swift.common.swob import Request, HTTPOk, HTTPCreated, HTTPAccepted, \
@@ -53,16 +53,19 @@ class TestStorletMiddlewareProxy(BaseTestStorletMiddleware):
             self.fail('Application loading got an error')
 
     def get_request_response(self, target, method, headers=None, body=None):
+        # Ensure the body is byte format on py3, this is needed until
+        # swift's Request supports byte format body when body is None in args
+        body = body or b''
         req = Request.blank(target, environ={'REQUEST_METHOD': method},
                             headers=headers, body=body)
         return self.get_response(req)
 
     def test_GET_without_storlets(self):
         def basic_get(path):
-            self.base_app.register('GET', path, HTTPOk, body='FAKE APP')
+            self.base_app.register('GET', path, HTTPOk, body=b'FAKE APP')
             resp = self.get_request_response(path, 'GET')
             self.assertEqual('200 OK', resp.status)
-            self.assertEqual('FAKE APP', resp.body)
+            self.assertEqual(b'FAKE APP', resp.body)
             self.base_app.reset_all()
 
         for target in ('AUTH_a', 'AUTH_a/c', 'AUTH_a/c/o'):
@@ -72,17 +75,17 @@ class TestStorletMiddlewareProxy(BaseTestStorletMiddleware):
     def test_GET_with_storlets(self):
         # TODO(takashi): decide request path based on config value
         target = '/v1/AUTH_a/c/o'
-        self.base_app.register('GET', target, HTTPOk, body='FAKE RESULT')
+        self.base_app.register('GET', target, HTTPOk, body=b'FAKE RESULT')
         storlet = '/v1/AUTH_a/storlet/Storlet-1.0.jar'
         self.base_app.register('GET', storlet, HTTPOk, headers={},
-                               body='jar binary')
+                               body=b'jar binary')
 
         acc_info = {'meta': {'storlet-enabled': 'true'}}
         with fake_acc_info(acc_info):
             headers = {'X-Run-Storlet': 'Storlet-1.0.jar'}
             resp = self.get_request_response(target, 'GET', headers=headers)
             self.assertEqual('200 OK', resp.status)
-            self.assertEqual('FAKE RESULT', resp.body)
+            self.assertEqual(b'FAKE RESULT', resp.body)
             calls = self.base_app.get_calls()
 
             # Make sure now we sent two requests to swift
@@ -92,7 +95,7 @@ class TestStorletMiddlewareProxy(BaseTestStorletMiddleware):
             self.assertEqual('HEAD', calls[0][0])
             self.assertEqual(storlet, calls[0][1])
 
-            # The last one is exexution GET call
+            # The last one is execution GET call
             self.assertEqual(target, calls[-1][1])
             self.assertIn('X-Run-Storlet', calls[-1][2])
 
@@ -112,7 +115,7 @@ class TestStorletMiddlewareProxy(BaseTestStorletMiddleware):
         target = '/v1/AUTH_a/c/o'
         self.base_app.register('GET', target, HTTPNotFound)
         storlet = '/v1/AUTH_a/storlet/Storlet-1.0.jar'
-        self.base_app.register('GET', storlet, HTTPOk, body='jar binary')
+        self.base_app.register('GET', storlet, HTTPOk, body=b'jar binary')
 
         with storlet_enabled():
             headers = {'X-Run-Storlet': 'Storlet-1.0.jar'}
@@ -133,9 +136,9 @@ class TestStorletMiddlewareProxy(BaseTestStorletMiddleware):
 
     def test_GET_with_storlets_and_storlet_range(self):
         target = '/v1/AUTH_a/c/o'
-        self.base_app.register('GET', target, HTTPOk, body='FAKE APP')
+        self.base_app.register('GET', target, HTTPOk, body=b'FAKE APP')
         storlet = '/v1/AUTH_a/storlet/Storlet-1.0.jar'
-        self.base_app.register('GET', storlet, HTTPOk, body='jar binary')
+        self.base_app.register('GET', storlet, HTTPOk, body=b'jar binary')
 
         with storlet_enabled():
             req_range = 'bytes=1-6'
@@ -144,11 +147,11 @@ class TestStorletMiddlewareProxy(BaseTestStorletMiddleware):
                        'X-Storlet-Range': req_range}
             resp = self.get_request_response(target, 'GET', headers=headers)
             self.assertEqual('200 OK', resp.status)
-            self.assertEqual('AKE AP', resp.body)
+            self.assertEqual(b'AKE AP', resp.body)
 
             self.assertNotIn('Content-Range', resp.headers)
-            self.assertEqual(resp.headers['Storlet-Input-Range'],
-                             'bytes 1-6/8')
+            self.assertEqual('bytes 1-6/8',
+                             resp.headers['Storlet-Input-Range'])
 
             raw_req = self.base_app.get_calls('GET', target)[0]
             for key in ['Range', 'X-Storlet-Range']:
@@ -158,9 +161,9 @@ class TestStorletMiddlewareProxy(BaseTestStorletMiddleware):
         # Create a single range request that needs to be
         # processed by the object handler
         target = '/v1/AUTH_a/c/o'
-        self.base_app.register('GET', target, HTTPOk, body='FAKE APP')
+        self.base_app.register('GET', target, HTTPOk, body=b'FAKE APP')
         storlet = '/v1/AUTH_a/storlet/Storlet-1.0.jar'
-        self.base_app.register('GET', storlet, HTTPOk, body='jar binary')
+        self.base_app.register('GET', storlet, HTTPOk, body=b'jar binary')
 
         with storlet_enabled():
             req_range = 'bytes=1-6'
@@ -170,49 +173,139 @@ class TestStorletMiddlewareProxy(BaseTestStorletMiddleware):
             # We assert that nothing actually happens
             # by the proxy handler
             self.assertEqual('200 OK', resp.status)
-            self.assertEqual('FAKE APP', resp.body)
+            self.assertEqual(b'FAKE APP', resp.body)
 
-    def test_GET_with_storlets_and_extra_resourece(self):
+    def test_GET_with_storlets_and_extra_resource(self):
         target = '/v1/AUTH_a/c/o'
-        self.base_app.register('GET', target, HTTPOk, body='FAKE APP')
+        self.base_app.register('GET', target, HTTPOk, body=b'FAKE APP')
         extra_target = '/v1/AUTH_a/c2/o2'
-        self.base_app.register('GET', extra_target, HTTPOk, body='Whooa')
+        self.base_app.register('GET', extra_target, HTTPOk, body=b'Whooa')
         storlet = '/v1/AUTH_a/storlet/Storlet-1.0.jar'
-        self.base_app.register('GET', storlet, HTTPOk, body='jar binary')
+        self.base_app.register('GET', storlet, HTTPOk, body=b'jar binary')
 
         with storlet_enabled():
             headers = {'X-Run-Storlet': 'Storlet-1.0.jar',
                        'X-Storlet-Extra-Resources': '/c2/o2'}
             resp = self.get_request_response(target, 'GET', headers=headers)
             self.assertEqual('200 OK', resp.status)
-            self.assertEqual('FAKE APP', resp.body)
+            self.assertEqual(b'FAKE APP', resp.body)
 
             # GET target called
             self.assertTrue(any(self.base_app.get_calls('GET', target)))
             # GET extra target also called
             self.assertTrue(any(self.base_app.get_calls('GET', extra_target)))
 
+    def test_PUT_with_storlets_and_extra_resource(self):
+        target = '/v1/AUTH_a/c/o'
+        self.base_app.register('PUT', target, HTTPCreated, body=b'')
+        extra_target = '/v1/AUTH_a/c2/o2'
+        self.base_app.register('GET', extra_target, HTTPOk, body=b'Whooa')
+        storlet = '/v1/AUTH_a/storlet/Storlet-1.0.jar'
+        self.base_app.register('GET', storlet, HTTPOk, body=b'jar binary')
+
+        with storlet_enabled():
+            headers = {'X-Run-Storlet': 'Storlet-1.0.jar',
+                       'X-Storlet-Extra-Resources': '/c2/o2'}
+            resp = self.get_request_response(target, 'PUT', headers=headers)
+            self.assertEqual('201 Created', resp.status)
+
+            # PUT target called
+            self.assertTrue(any(self.base_app.get_calls('PUT', target)))
+            # GET extra target also called
+            self.assertTrue(any(self.base_app.get_calls('GET', extra_target)))
+
+    def test_GET_with_storlets_and_extra_resource_within_limit(self):
+        target = '/v1/AUTH_a/c/o'
+        self.base_app.register('GET', target, HTTPOk, body=b'FAKE APP')
+        extra_targets = ['/v1/AUTH_a/c2/o2', '/v1/AUTH_a/c3/o3']
+        for extra_target in extra_targets:
+            self.base_app.register('GET', extra_target, HTTPOk, body=b'foo')
+        storlet = '/v1/AUTH_a/storlet/Storlet-1.0.jar'
+        self.base_app.register('GET', storlet, HTTPOk, body=b'jar binary')
+
+        self.conf['max_extra_resources'] = 2
+
+        with storlet_enabled():
+            headers = {'X-Run-Storlet': 'Storlet-1.0.jar',
+                       'X-Storlet-Extra-Resources': '/c2/o2,/c3/o3'}
+            resp = self.get_request_response(target, 'GET', headers=headers)
+            self.assertEqual('200 OK', resp.status)
+            self.assertEqual(b'FAKE APP', resp.body)
+
+            # GET target called
+            self.assertTrue(any(self.base_app.get_calls('GET', target)))
+            # GET extra targets also called
+            for extra_target in extra_targets:
+                self.assertTrue(
+                    any(self.base_app.get_calls('GET', extra_target)))
+
+    def test_GET_with_storlets_and_extra_resource_not_object(self):
+        target = '/v1/AUTH_a/c/o'
+
+        with storlet_enabled():
+            headers = {'X-Run-Storlet': 'Storlet-1.0.jar',
+                       'X-Storlet-Extra-Resources': '/c2o2'}
+            resp = self.get_request_response(target, 'GET', headers=headers)
+            self.assertEqual('400 Bad Request', resp.status)
+
+    def test_GET_with_storlets_and_extra_resource_over_limit(self):
+        target = '/v1/AUTH_a/c/o'
+        self.conf['max_extra_resources'] = 1
+
+        with storlet_enabled():
+            headers = {'X-Run-Storlet': 'Storlet-1.0.jar',
+                       'X-Storlet-Extra-Resources': '/c2/o2,/c3/o3'}
+            resp = self.get_request_response(target, 'GET', headers=headers)
+            self.assertEqual('400 Bad Request', resp.status)
+
+    def test_GET_with_storlets_and_extra_resource_disabled(self):
+        target = '/v1/AUTH_a/c/o'
+        self.conf['max_extra_resources'] = 0
+
+        with storlet_enabled():
+            headers = {'X-Run-Storlet': 'Storlet-1.0.jar',
+                       'X-Storlet-Extra-Resources': '/c2/o2'}
+            resp = self.get_request_response(target, 'GET', headers=headers)
+            self.assertEqual('403 Forbidden', resp.status)
+
     def test_GET_slo_without_storlets(self):
         target = '/v1/AUTH_a/c/slo_manifest'
         self.base_app.register('GET', target, HTTPOk,
                                headers={'x-static-large-object': 'True'},
-                               body='FAKE APP')
+                               body=b'FAKE APP')
         resp = self.get_request_response(target, 'GET')
-        self.assertEqual('FAKE APP', resp.body)
+        self.assertEqual(b'FAKE APP', resp.body)
 
     def test_GET_slo_with_storlets(self):
         target = '/v1/AUTH_a/c/slo_manifest'
         self.base_app.register('GET', target, HTTPOk,
                                headers={'x-static-large-object': 'True'},
-                               body='FAKE APP')
+                               body=b'FAKE APP')
         storlet = '/v1/AUTH_a/storlet/Storlet-1.0.jar'
-        self.base_app.register('GET', storlet, HTTPOk, body='jar binary')
+        self.base_app.register('GET', storlet, HTTPOk, body=b'jar binary')
 
         with storlet_enabled():
             headers = {'X-Run-Storlet': 'Storlet-1.0.jar'}
             resp = self.get_request_response(target, 'GET', headers=headers)
             self.assertEqual('200 OK', resp.status)
-            self.assertEqual('FAKE APP', resp.body)
+            self.assertEqual(b'FAKE APP', resp.body)
+
+            calls = self.base_app.get_calls()
+            self.assertEqual(2, len(calls))
+
+    def test_GET_symlink_with_storlets(self):
+        target = '/v1/AUTH_a/c/o'
+        self.base_app.register('GET', target, HTTPOk,
+                               headers={'x-symlink-target': 'c/o2'},
+                               body=b'FAKE APP')
+        storlet = '/v1/AUTH_a/storlet/Storlet-1.0.jar'
+        self.base_app.register('GET', storlet, HTTPOk, body=b'jar binary')
+
+        with storlet_enabled():
+            headers = {'X-Run-Storlet': 'Storlet-1.0.jar'}
+            resp = self.get_request_response(target, 'GET', headers=headers)
+            self.assertEqual('200 OK', resp.status)
+            self.assertEqual(b'FAKE APP', resp.body)
 
             calls = self.base_app.get_calls()
             self.assertEqual(2, len(calls))
@@ -220,12 +313,12 @@ class TestStorletMiddlewareProxy(BaseTestStorletMiddleware):
     def test_GET_with_storlets_no_object(self):
         target = '/v1/AUTH_a/c/'
         self.base_app.register('GET', target, HTTPOk,
-                               body='FAKE APP')
+                               body=b'FAKE APP')
         with storlet_enabled():
             headers = {'X-Run-Storlet': 'Storlet-1.0.jar'}
             resp = self.get_request_response(target, 'GET', headers=headers)
             self.assertEqual('200 OK', resp.status)
-            self.assertEqual('FAKE APP', resp.body)
+            self.assertEqual(b'FAKE APP', resp.body)
 
             # Make sure now we sent one request to swift
             # that is, storlets path was not executed
@@ -246,14 +339,14 @@ class TestStorletMiddlewareProxy(BaseTestStorletMiddleware):
 
     def test_PUT_with_storlets(self):
         target = '/v1/AUTH_a/c/o'
-        self.base_app.register('PUT', target, HTTPCreated)
+        self.base_app.register('PUT', target, HTTPCreated, body=b'')
         storlet = '/v1/AUTH_a/storlet/Storlet-1.0.jar'
-        self.base_app.register('GET', storlet, HTTPOk, body='jar binary')
+        self.base_app.register('GET', storlet, HTTPOk, body=b'jar binary')
 
         with storlet_enabled():
             headers = {'X-Run-Storlet': 'Storlet-1.0.jar'}
             resp = self.get_request_response(target, 'PUT', headers=headers,
-                                             body='FAKE APP')
+                                             body=b'FAKE APP')
             self.assertEqual('201 Created', resp.status)
 
             calls = self.base_app.get_calls()
@@ -267,7 +360,7 @@ class TestStorletMiddlewareProxy(BaseTestStorletMiddleware):
             # The last one is PUT request about processed object
             self.assertEqual('PUT', calls[-1][0])
             self.assertEqual(target, calls[-1][1])
-            self.assertEqual(calls[-1][3], 'FAKE APP')
+            self.assertEqual(b'FAKE APP', calls[-1][3])
 
     def test_PUT_with_storlets_no_object(self):
         target = '/v1/AUTH_a/c/'
@@ -275,7 +368,7 @@ class TestStorletMiddlewareProxy(BaseTestStorletMiddleware):
         with storlet_enabled():
             headers = {'X-Run-Storlet': 'Storlet-1.0.jar'}
             resp = self.get_request_response(target, 'PUT', headers=headers,
-                                             body='FAKE APP')
+                                             body=b'FAKE APP')
             self.assertEqual('201 Created', resp.status)
 
             calls = self.base_app.get_calls()
@@ -298,10 +391,12 @@ class TestStorletMiddlewareProxy(BaseTestStorletMiddleware):
         source = '/v1/AUTH_a/c/so'
         target = '/v1/AUTH_a/c/to'
         copy_from = 'c/so'
-        self.base_app.register('GET', source, HTTPOk, body='source body')
+        self.base_app.register('GET', source, HTTPOk,
+                               headers={'x-object-meta-name': 'name'},
+                               body=b'source body')
         self.base_app.register('PUT', target, HTTPCreated)
         storlet = '/v1/AUTH_a/storlet/Storlet-1.0.jar'
-        self.base_app.register('GET', storlet, HTTPOk, body='jar binary')
+        self.base_app.register('GET', storlet, HTTPOk, body=b'jar binary')
 
         with storlet_enabled():
             headers = {'X-Copy-From': copy_from,
@@ -310,12 +405,55 @@ class TestStorletMiddlewareProxy(BaseTestStorletMiddleware):
             resp = self.get_request_response(target, 'PUT', headers=headers)
             self.assertEqual('201 Created', resp.status)
             get_calls = self.base_app.get_calls('GET', source)
-            self.assertEqual(len(get_calls), 1)
-            self.assertEqual(get_calls[-1][3], '')
-            self.assertEqual(get_calls[-1][1], source)
+            self.assertEqual(1, len(get_calls))
+            self.assertEqual(b'', get_calls[-1][3])
+            self.assertEqual(source, get_calls[-1][1])
+            self.assertIn('X-Run-Storlet', get_calls[-1][2])
             put_calls = self.base_app.get_calls('PUT', target)
-            self.assertEqual(len(put_calls), 1)
-            self.assertEqual(put_calls[-1][3], 'source body')
+            self.assertEqual(1, len(put_calls))
+            self.assertEqual(b'source body', put_calls[-1][3])
+            self.assertIn('X-Object-Meta-Name', dict(put_calls[-1][2]))
+            self.assertEqual('name', put_calls[-1][2]['X-Object-Meta-Name'])
+            self.assertNotIn('X-Run-Storlet', put_calls[-1][2])
+            # no invocation (at gateway stub) at proxy
+            for debug_line in self.logger.get_log_lines('debug'):
+                self.assertNotIn("Identity invocation is called", debug_line)
+
+    def test_PUT_copy_with_storlets_run_on_proxy(self):
+        source = '/v1/AUTH_a/c/so'
+        target = '/v1/AUTH_a/c/to'
+        copy_from = 'c/so'
+        self.base_app.register('GET', source, HTTPOk,
+                               headers={'x-object-meta-name': 'name'},
+                               body=b'source body')
+        self.base_app.register('PUT', target, HTTPCreated)
+        storlet = '/v1/AUTH_a/storlet/Storlet-1.0.jar'
+        self.base_app.register('GET', storlet, HTTPOk, body=b'jar binary')
+
+        with storlet_enabled():
+            headers = {'X-Copy-From': copy_from,
+                       'X-Run-Storlet': 'Storlet-1.0.jar',
+                       'X-Storlet-Run-On-Proxy': '',
+                       'X-Backend-Storage-Policy-Index': 0}
+            resp = self.get_request_response(target, 'PUT', headers=headers)
+            self.assertEqual('201 Created', resp.status)
+            get_calls = self.base_app.get_calls('GET', source)
+            self.assertEqual(1, len(get_calls))
+            self.assertEqual(b'', get_calls[-1][3])
+            self.assertEqual(source, get_calls[-1][1])
+            self.assertNotIn('X-Run-Storlet', get_calls[-1][2])
+            put_calls = self.base_app.get_calls('PUT', target)
+            self.assertEqual(1, len(put_calls))
+            self.assertEqual(b'source body', put_calls[-1][3])
+            self.assertIn('X-Object-Meta-Name', dict(put_calls[-1][2]))
+            self.assertEqual('name', put_calls[-1][2]['X-Object-Meta-Name'])
+            self.assertNotIn('X-Run-Storlet', put_calls[-1][2])
+            # no invocation at proxy
+            for debug_line in self.logger.get_log_lines('debug'):
+                if "Identity invocation is called" in debug_line:
+                    break
+            else:
+                self.fail('no invocation message found at proxy')
 
     def test_COPY_verb_without_storlets(self):
         source = '/v1/AUTH_a/c/so'
@@ -331,10 +469,12 @@ class TestStorletMiddlewareProxy(BaseTestStorletMiddleware):
         source = '/v1/AUTH_a/c/so'
         target = '/v1/AUTH_a/c/to'
         destination = 'c/to'
-        self.base_app.register('GET', source, HTTPOk, body='source body')
+        self.base_app.register('GET', source, HTTPOk,
+                               headers={'x-object-meta-name': 'name'},
+                               body=b'source body')
         self.base_app.register('PUT', target, HTTPCreated)
         storlet = '/v1/AUTH_a/storlet/Storlet-1.0.jar'
-        self.base_app.register('GET', storlet, HTTPOk, body='jar binary')
+        self.base_app.register('GET', storlet, HTTPOk, body=b'jar binary')
 
         with storlet_enabled():
             headers = {'Destination': destination,
@@ -343,17 +483,58 @@ class TestStorletMiddlewareProxy(BaseTestStorletMiddleware):
             resp = self.get_request_response(source, 'COPY', headers=headers)
             self.assertEqual('201 Created', resp.status)
             get_calls = self.base_app.get_calls('GET', source)
-            self.assertEqual(len(get_calls), 1)
-            self.assertEqual(get_calls[-1][3], '')
-            self.assertEqual(get_calls[-1][1], source)
+            self.assertEqual(1, len(get_calls))
+            self.assertEqual(b'', get_calls[-1][3])
+            self.assertEqual(source, get_calls[-1][1])
+            self.assertIn('X-Run-Storlet', get_calls[-1][2])
             put_calls = self.base_app.get_calls('PUT', target)
-            self.assertEqual(len(put_calls), 1)
-            self.assertEqual(put_calls[-1][3], 'source body')
+            self.assertEqual(1, len(put_calls))
+            self.assertEqual(b'source body', put_calls[-1][3])
+            self.assertEqual('name', put_calls[-1][2]['X-Object-Meta-Name'])
+            self.assertNotIn('X-Run-Storlet', put_calls[-1][2])
+            # no invocation at proxy
+            for debug_line in self.logger.get_log_lines('debug'):
+                self.assertNotIn("Identity invocation is called", debug_line)
+
+    def test_COPY_verb_with_storlets_run_on_porxy(self):
+        source = '/v1/AUTH_a/c/so'
+        target = '/v1/AUTH_a/c/to'
+        destination = 'c/to'
+        self.base_app.register('GET', source, HTTPOk,
+                               headers={'x-object-meta-name': 'name'},
+                               body=b'source body')
+        self.base_app.register('PUT', target, HTTPCreated)
+        storlet = '/v1/AUTH_a/storlet/Storlet-1.0.jar'
+        self.base_app.register('GET', storlet, HTTPOk, body=b'jar binary')
+
+        with storlet_enabled():
+            headers = {'Destination': destination,
+                       'X-Run-Storlet': 'Storlet-1.0.jar',
+                       'X-Storlet-Run-On-Proxy': '',
+                       'X-Backend-Storage-Policy-Index': 0}
+            resp = self.get_request_response(source, 'COPY', headers=headers)
+            self.assertEqual('201 Created', resp.status)
+            get_calls = self.base_app.get_calls('GET', source)
+            self.assertEqual(1, len(get_calls))
+            self.assertEqual(b'', get_calls[-1][3])
+            self.assertEqual(source, get_calls[-1][1])
+            self.assertNotIn('X-Run-Storlet', get_calls[-1][2])
+            put_calls = self.base_app.get_calls('PUT', target)
+            self.assertEqual(1, len(put_calls))
+            self.assertEqual(b'source body', put_calls[-1][3])
+            self.assertEqual('name', put_calls[-1][2]['X-Object-Meta-Name'])
+            self.assertNotIn('X-Run-Storlet', put_calls[-1][2])
+            # no invocation at proxy
+            for debug_line in self.logger.get_log_lines('debug'):
+                if "Identity invocation is called" in debug_line:
+                    break
+            else:
+                self.fail('no invocation message found at proxy')
 
     def test_copy_with_unsupported_headers(self):
         target = '/v1/AUTH_a/c/o'
         storlet = '/v1/AUTH_a/storlet/Storlet-1.0.jar'
-        self.base_app.register('GET', storlet, HTTPOk, body='jar binary')
+        self.base_app.register('GET', storlet, HTTPOk, body=b'jar binary')
 
         def copy_400(method, copy_header):
             base_headers = {'X-Run-Storlet': 'Storlet-1.0.jar',
@@ -435,12 +616,12 @@ class TestStorletMiddlewareProxy(BaseTestStorletMiddleware):
                     'X-Object-Meta-Storlet-Main':
                         'org.openstack.storlet.Storlet'}
         self.base_app.register('GET', target, HTTPOk, headers=sheaders,
-                               body='jar binary')
+                               body=b'jar binary')
 
         with storlet_enabled():
             resp = self.get_request_response(target, 'GET')
             self.assertEqual('200 OK', resp.status)
-            self.assertEqual('jar binary', resp.body)
+            self.assertEqual(b'jar binary', resp.body)
             for key in sheaders:
                 self.assertEqual(sheaders[key], resp.headers.get(key))
 
@@ -469,12 +650,12 @@ class TestStorletMiddlewareProxy(BaseTestStorletMiddleware):
         sheaders = {'X-Object-Meta-Storlet-Dependency-Version': '1',
                     'X-Object-Meta-Storlet-Dependency-Permissions': '0755'}
         self.base_app.register('GET', target, HTTPOk, headers=sheaders,
-                               body='FAKE APP')
+                               body=b'FAKE APP')
 
         with storlet_enabled():
             resp = self.get_request_response(target, 'GET')
             self.assertEqual('200 OK', resp.status)
-            self.assertEqual('FAKE APP', resp.body)
+            self.assertEqual(b'FAKE APP', resp.body)
             for key in sheaders:
                 self.assertEqual(sheaders[key], resp.headers.get(key))
 
@@ -491,7 +672,7 @@ class TestStorletMiddlewareProxy(BaseTestStorletMiddleware):
         headers = {'X-Storlet-Container-Read': 'aa',
                    'X-Storlet-Name': 'bb'}
         target = '/v1/AUTH_a/c/o'
-        self.base_app.register('POST', target, HTTPNoContent, body='')
+        self.base_app.register('POST', target, HTTPNoContent, body=b'')
         with storlet_enabled():
             resp = self.get_request_response(target, 'POST', headers=headers)
             self.assertEqual('204 No Content', resp.status)
@@ -503,9 +684,8 @@ class TestStorletMiddlewareProxy(BaseTestStorletMiddleware):
                 resp = self.get_request_response(target, 'POST',
                                                  headers=headers)
                 self.assertEqual('400 Bad Request', resp.status)
-                self.assertEqual(('storlet ACL update cannot be'
-                                  ' a storlet container'),
-                                 resp.body)
+                msg = b'storlet ACL update cannot be a storlet container'
+                self.assertEqual(msg, resp.body)
 
         # Test wrong headers content
         hlist = [{'X-Storlet-Container-Read': ''},
@@ -518,7 +698,7 @@ class TestStorletMiddlewareProxy(BaseTestStorletMiddleware):
                 resp = self.get_request_response(target, 'POST',
                                                  headers=headers)
                 self.assertEqual('400 Bad Request', resp.status)
-                self.assertTrue('missing a mandatory header' in resp.body)
+                self.assertIn(b'missing a mandatory header', resp.body)
 
         # Test wrong ACL content
         hlist = [{'X-Storlet-Container-Read': 'a,b',
@@ -531,7 +711,7 @@ class TestStorletMiddlewareProxy(BaseTestStorletMiddleware):
                 resp = self.get_request_response(target, 'POST',
                                                  headers=headers)
                 self.assertEqual('400 Bad Request', resp.status)
-                self.assertTrue('mulformed storlet or user name' in resp.body)
+                self.assertIn(b'mulformed storlet or user name', resp.body)
 
     def test_POST_storlet_acl(self):
         target = '/v1/AUTH_a/c'
@@ -546,11 +726,11 @@ class TestStorletMiddlewareProxy(BaseTestStorletMiddleware):
             resp = self.get_request_response(target, 'POST', headers=headers)
             self.assertEqual('204 No Content', resp.status)
             head_calls = self.base_app.get_calls('HEAD', target)
-            self.assertEqual(len(head_calls), 1)
+            self.assertEqual(1, len(head_calls))
             post_calls = self.base_app.get_calls('POST', target)
-            self.assertEqual(len(post_calls), 1)
-            self.assertEqual(post_calls[-1][2]['X-Container-Read'],
-                             expected_read_acl)
+            self.assertEqual(1, len(post_calls))
+            self.assertEqual(expected_read_acl,
+                             post_calls[-1][2]['X-Container-Read'])
 
     def test_GET_with_invalid_referrer(self):
         headers = [{'Referer': REFERER_PREFIX},
@@ -631,9 +811,39 @@ class TestStorletProxyHandler(unittest.TestCase):
         handler._remove_storlet_headers(headers)
 
         self.assertNotIn('X-Storlet-Key1', headers)
-        self.assertEqual(headers['X-Key2'], 'Value2')
+        self.assertEqual('Value2', headers['X-Key2'])
         self.assertNotIn('X-Object-Meta-Storlet-Key3', headers)
-        self.assertEqual(headers['X-Object-Meta-Key4'], 'Value4')
+        self.assertEqual('Value4', headers['X-Object-Meta-Key4'])
+
+    def test_get_storlet_invocation_options(self):
+        req = Request.blank(
+            '/v1/acc/cont/obj',
+            environ={'REQUEST_METHOD': 'GET'},
+            headers={'X-Run-Storlet': 'Storlet-1.0.jar',
+                     'X-Storlet-Foo': 'baa'})
+        with storlet_enabled():
+            handler = self.handler_class(
+                req, self.conf, self.gateway_conf, mock.MagicMock(),
+                mock.MagicMock())
+
+        options = handler._get_storlet_invocation_options(req)
+        self.assertEqual('baa', options['storlet_foo'])
+        self.assertFalse(options['generate_log'])
+
+        req = Request.blank(
+            '/v1/acc/cont/obj',
+            environ={'REQUEST_METHOD': 'GET'},
+            headers={'X-Run-Storlet': 'Storlet-1.0.jar',
+                     'X-Storlet-Foo': 'baa',
+                     'X-Storlet-Generate-Log': 'True'})
+        with storlet_enabled():
+            handler = self.handler_class(
+                req, self.conf, self.gateway_conf, mock.MagicMock(),
+                mock.MagicMock())
+
+        options = handler._get_storlet_invocation_options(req)
+        self.assertEqual('baa', options['storlet_foo'])
+        self.assertTrue(options['generate_log'])
 
 
 if __name__ == '__main__':
